@@ -11,14 +11,14 @@ use {
     },
 };
 
-/// TUI Application state, containing a drawer state.
-///
-/// Needs a closet
+/// TUI Application state
 pub struct AppState {
     pub open_closet: OpenCloset,
     pub drawer_state: DrawerState,
     // the help state, if help is currently displayed
     pub help: Option<HelpState>,
+    // the menu, if any
+    pub menu: Option<MenuState>,
     pub message: Option<Message>,
     pub hide_values: bool,
     // number of drawers created during this session
@@ -32,6 +32,7 @@ impl AppState {
             open_closet,
             drawer_state: DrawerState::NoneOpen,
             help: None,
+            menu: None,
             message: None,
             hide_values: args.hide,
             created_drawers: 0,
@@ -211,8 +212,7 @@ impl AppState {
             match kind {
                 MouseEventKind::Up(MouseButton::Left) => {
                     if modifiers == KeyModifiers::NONE {
-                        debug!("clicked line: {:?}", des.clicked_line(row));
-                        if let Some(clicked_line) = des.clicked_line(row) {
+                        if let Some(clicked_line) = des.clicked_line(row as usize) {
                             use DrawerFocus::*;
                             // if we're here we know the clicked input isn't focused
                             let in_name = des.layout().is_in_name_column(column);
@@ -257,6 +257,243 @@ impl AppState {
         Ok(())
     }
 
+    /// delete entry (with confirmation)
+    fn propose_entry_removal(&mut self) {
+        if let DrawerState::DrawerEdit(des) = &mut self.drawer_state {
+            if let Some(line) = des.focus.line() {
+                des.focus = DrawerFocus::PendingRemoval { line };
+                let mut menu = MenuState::default();
+                menu.actions.push(Action::ConfirmEntryRemoval);
+                menu.actions.push(Action::Back);
+                menu.selection = 1;
+                self.menu = Some(menu);
+            }
+        }
+    }
+    fn cancel_entry_removal(&mut self) {
+        if let DrawerState::DrawerEdit(des) = &mut self.drawer_state {
+            if let DrawerFocus::PendingRemoval { line } = &des.focus {
+                let line = *line;
+                des.focus = DrawerFocus::NameSelected { line };
+                self.menu = None;
+                self.help = None;
+            }
+        }
+    }
+
+    pub fn on_action(&mut self, action: Action) -> Result<CmdResult, SafeClosetError> {
+        use {
+            DrawerFocus::*,
+            DrawerState::*,
+        };
+        debug!("executing action {:?}", action);
+        match action {
+            Action::Back => {
+                if self.drawer_state.is_pending_removal() {
+                    self.cancel_entry_removal();
+                } else if self.menu.is_some() {
+                    self.menu = None;
+                } else if self.help.is_some() {
+                    self.help = None;
+                } else if matches!(self.drawer_state, DrawerCreation(_) | DrawerOpening(_)) {
+                    self.drawer_state = NoneOpen;
+                } else if self.close_drawer_input(true) {
+                    debug!("closing drawer input");
+                } else {
+                    debug!("opening menu");
+                    let mut menu = MenuState::default();
+                    self.fill_menu(&mut menu.actions);
+                    self.menu = Some(menu);
+                }
+            }
+            Action::NewDrawer => {
+                self.help = None;
+                self.menu = None;
+                self.push_back_drawer()?;
+                self.drawer_state = DrawerCreation(PasswordInputState::new(false));
+            }
+            Action::OpenDrawer => {
+                self.help = None;
+                self.menu = None;
+                self.push_back_drawer()?;
+                self.drawer_state = DrawerOpening(PasswordInputState::new(true));
+            }
+            Action::SaveDrawer => {
+                if self.drawer_state.is_edit() {
+                    self.help = None;
+                    self.menu = None;
+                    debug!("user requests save, keep state");
+                    self.save(true)?;
+                } else {
+                    self.set_error("no open drawer");
+                }
+            }
+            Action::CloseDrawer => {
+                self.help = None;
+                self.menu = None;
+                self.save(true)?;
+                self.push_back_drawer()?;
+                let _ = self.open_closet.close_deepest_drawer();
+                self.drawer_state = match self.open_closet.take_deepest_open_drawer() {
+                    Some(open_drawer) => DrawerState::edit(open_drawer),
+                    None => DrawerState::NoneOpen,
+                };
+            }
+            Action::Help => {
+                self.help = Some(HelpState::default());
+                self.menu = None;
+            }
+            Action::Quit => {
+                debug!("user requests quit");
+                return Ok(CmdResult::Quit);
+            }
+            Action::MoveLineUp => {
+                if let DrawerEdit(des) = &mut self.drawer_state {
+                    let entries = &mut des.drawer.content.entries;
+                    let len = entries.len();
+                    match &mut des.focus {
+                        NameSelected { line } => {
+                            let new_line = (*line + len - 1) % len;
+                            entries.swap(*line, new_line);
+                            des.focus = NameSelected { line: new_line };
+                        }
+                        ValueSelected { line } => {
+                            let new_line = (*line + len - 1) % len;
+                            entries.swap(*line, new_line);
+                            des.focus = ValueSelected { line: new_line };
+                        }
+                        ValueEdit { input, .. }  => {
+                            input.move_current_line_up();
+                        }
+                        _ => {}
+                    }
+                    des.update_search();
+                }
+            }
+            Action::MoveLineDown => {
+                if let DrawerEdit(des) = &mut self.drawer_state {
+                    let entries = &mut des.drawer.content.entries;
+                    let len = entries.len();
+                    match &mut des.focus {
+                        NameSelected { line } => {
+                            let new_line = (*line + 1) % len;
+                            entries.swap(*line, new_line);
+                            des.focus = NameSelected { line: new_line };
+                        }
+                        ValueSelected { line } => {
+                            let new_line = (*line + 1) % len;
+                            entries.swap(*line, new_line);
+                            des.focus = ValueSelected { line: new_line };
+                        }
+                        ValueEdit { input, .. }  => {
+                            input.move_current_line_down();
+                        }
+                        _ => {}
+                    }
+                    des.update_search();
+                }
+            }
+            Action::ToggleHiding => {
+                // toggle visibility of password or values
+                self.help = None;
+                self.menu = None;
+                if let DrawerCreation(pis) | DrawerOpening(pis) = &mut self.drawer_state {
+                    pis.input.password_mode ^= true;
+                    return Ok(CmdResult::Stay);
+                }
+                if let DrawerEdit(des) = &mut self.drawer_state {
+                    des.drawer.content.settings.hide_values ^= true;
+                    return Ok(CmdResult::Stay);
+                }
+            }
+            Action::OpenAllValues | Action::CloseAllValues=> {
+                self.help = None;
+                self.menu = None;
+                if let DrawerEdit(des) = &mut self.drawer_state {
+                    des.drawer.content.settings.open_all_values ^= true;
+                    return Ok(CmdResult::Stay);
+                }
+            }
+            Action::Copy => {
+                self.help = None;
+                self.menu = None;
+                self.copy();
+            }
+            Action::Cut => {
+                self.help = None;
+                self.menu = None;
+                self.cut();
+            }
+            Action::Paste => {
+                self.help = None;
+                self.menu = None;
+                self.paste();
+            }
+            Action::ConfirmEntryRemoval => {
+                self.help = None;
+                self.menu = None;
+                info!("user requests entry removal");
+                if let DrawerEdit(des) = &mut self.drawer_state {
+                    if let PendingRemoval { line } = &des.focus {
+                        let line = *line;
+                        if let Some(idx) = des.listed_entry_idx(line) {
+                            // we either confirm (delete) or cancel removal
+                            des.drawer.content.entries.remove(idx);
+                            des.focus = if line > 0 {
+                                NameSelected { line: line - 1 }
+                            } else {
+                                NoneSelected
+                            };
+                            des.update_search();
+                        }
+                    }
+                }
+            }
+            Action::NewEntry => {
+                if let DrawerEdit(des) = &mut self.drawer_state {
+                    self.help = None;
+                    self.menu = None;
+                    des.search.clear();
+                    let idx = des.drawer.content.empty_entry();
+                    des.edit_entry_name_by_line(idx, EditionPos::Start);
+                }
+            }
+            Action::RemoveLine => {
+                self.propose_entry_removal();
+            }
+            Action::Search => {
+                if let DrawerEdit(des) = &mut self.drawer_state {
+                    if let Some(line) = des.focus.line() {
+                        des.search.set_best_line(line);
+                    }
+                    let previous_idx = des.focus
+                        .line()
+                        .and_then(|line| des.listed_entry_idx(line));
+                    des.focus = SearchEdit { previous_idx };
+                }
+            }
+        }
+        Ok(CmdResult::Stay)
+    }
+
+    /// Add the relevant possible actions to the menu
+    pub fn fill_menu(&self, actions: &mut Vec<Action>) {
+        actions.push(Action::Back);
+        actions.push(Action::NewDrawer);
+        actions.push(Action::OpenDrawer);
+        if let DrawerState::DrawerEdit(des) = &self.drawer_state {
+            actions.push(Action::SaveDrawer);
+            actions.push(Action::CloseDrawer);
+            actions.push(Action::ToggleHiding);
+            if des.drawer.content.settings.open_all_values {
+                actions.push(Action::CloseAllValues);
+            } else {
+                actions.push(Action::OpenAllValues);
+            }
+        }
+        actions.push(Action::Help);
+        actions.push(Action::Quit);
+    }
 
     /// Handle a key event
     pub fn on_key(&mut self, key: KeyEvent) -> Result<CmdResult, SafeClosetError> {
@@ -266,176 +503,31 @@ impl AppState {
         };
         self.message = None;
 
-        // We start with the few actions that can be done the same with or
-        // without the help screen displayed
-
-        if key == CONTROL_N { // new drawer
-            self.push_back_drawer()?;
-            self.drawer_state = DrawerCreation(PasswordInputState::new(false));
-            return Ok(CmdResult::Stay);
-        }
-
-        if key == CONTROL_O { // open drawer
-            self.help = None;
-            self.push_back_drawer()?;
-            self.drawer_state = DrawerOpening(PasswordInputState::new(true));
-            return Ok(CmdResult::Stay);
-        }
-
-        if key == CONTROL_Q {
-            debug!("user requests quit");
-            return Ok(CmdResult::Quit);
-        }
-
-        if key == CONTROL_S {
-            debug!("user requests save, keep state");
-            self.save(true)?;
-            return Ok(CmdResult::Stay);
-        }
-
-        // if key == CONTROL_X {
-        //     debug!("user requests save and quit");
-        //     self.save(false)?;
-        //     return Ok(CmdResult::Quit);
-        // }
-
-        if key == CONTROL_U { // up the drawer chain, close the current one
-            if self.help.is_some() {
-                // close the help
-                self.help = None;
-            } else {
-                // close the drawer
-                self.save(true)?;
-                self.push_back_drawer()?;
-                let _ = self.open_closet.close_deepest_drawer();
-                self.drawer_state = match self.open_closet.take_deepest_open_drawer() {
-                    Some(open_drawer) => DrawerState::edit(open_drawer),
-                    None => DrawerState::NoneOpen,
-                };
-            }
-            return Ok(CmdResult::Stay);
-        }
-
-        if key == CONTROL_C {
-            self.copy();
-            return Ok(CmdResult::Stay);
-        }
-
-        if key == CONTROL_V {
-            self.paste();
-            return Ok(CmdResult::Stay);
-        }
-
-        if key == CONTROL_X {
-            self.cut();
-            return Ok(CmdResult::Stay);
-        }
-
-        if key == ESC {
-            if self.help.is_some() {
-                self.help = None;
-            } else if matches!(self.drawer_state, DrawerCreation(_) | DrawerOpening(_)) {
-                self.drawer_state = NoneOpen;
-            } else if let DrawerEdit(des) = &mut self.drawer_state {
-                if !des.close_input(true) {
-                    des.focus = NoneSelected;
+        if let Some(input) = self.drawer_state.input() {
+            if input.apply_key_event(key) {
+                if let DrawerEdit(des) = &mut self.drawer_state {
+                    if des.focus.is_search() {
+                        des.search.update(&des.drawer);
+                    }
                 }
+                return Ok(CmdResult::Stay);
             }
-            return Ok(CmdResult::Stay);
         }
 
-        // If the help is shown, it captures other events
+        if let Some(action) = Action::for_key(key) {
+            return self.on_action(action);
+        }
+
+        if let Some(menu_state) = self.menu.as_mut() {
+            return menu_state.on_key(key)
+                .map_or(Ok(CmdResult::Stay), |a| self.on_action(a));
+        }
+
         if let Some(help_state) = &mut self.help {
             help_state.apply_key_event(key);
             return Ok(CmdResult::Stay);
         }
 
-        if key == CONTROL_UP { // moving the selected line up
-            if let DrawerEdit(des) = &mut self.drawer_state {
-                let entries = &mut des.drawer.content.entries;
-                let len = entries.len();
-                match &mut des.focus {
-                    NameSelected { line } => {
-                        let new_line = (*line + len - 1) % len;
-                        entries.swap(*line, new_line);
-                        des.focus = NameSelected { line: new_line };
-                    }
-                    ValueSelected { line } => {
-                        let new_line = (*line + len - 1) % len;
-                        entries.swap(*line, new_line);
-                        des.focus = ValueSelected { line: new_line };
-                    }
-                    ValueEdit { input, .. }  => {
-                        input.move_current_line_up();
-                    }
-                    _ => {}
-                }
-                des.update_search();
-            }
-            return Ok(CmdResult::Stay);
-        }
-        if key == CONTROL_DOWN { // moving the selected line down
-            if let DrawerEdit(des) = &mut self.drawer_state {
-                let entries = &mut des.drawer.content.entries;
-                let len = entries.len();
-                match &mut des.focus {
-                    NameSelected { line } => {
-                        let new_line = (*line + 1) % len;
-                        entries.swap(*line, new_line);
-                        des.focus = NameSelected { line: new_line };
-                    }
-                    ValueSelected { line } => {
-                        let new_line = (*line + 1) % len;
-                        entries.swap(*line, new_line);
-                        des.focus = ValueSelected { line: new_line };
-                    }
-                    ValueEdit { input, .. }  => {
-                        input.move_current_line_down();
-                    }
-                    _ => {}
-                }
-                des.update_search();
-            }
-            return Ok(CmdResult::Stay);
-        }
-
-        if let DrawerEdit(des) = &mut self.drawer_state {
-            // -- pending removal
-            if let PendingRemoval { line } = &des.focus {
-                let line = *line;
-                if let Some(idx) = des.listed_entry_idx(line) {
-                    // we either confirm (delete) or cancel removal
-                    if as_letter(key) == Some('y') {
-                        info!("user requests entry removal");
-                        des.drawer.content.entries.remove(idx);
-                        des.focus = if line > 0 {
-                            NameSelected { line: line - 1 }
-                        } else {
-                            NoneSelected
-                        };
-                        des.update_search();
-                    } else {
-                        info!("user cancels entry removal");
-                        des.focus = NameSelected { line };
-                    }
-                }
-                return Ok(CmdResult::Stay);
-            }
-        }
-
-        // -- toggle visibility of password or values
-
-        if key == CONTROL_H {
-            if let DrawerCreation(pis) | DrawerOpening(pis) = &mut self.drawer_state {
-                pis.input.password_mode ^= true;
-                return Ok(CmdResult::Stay);
-            }
-            if let DrawerEdit(des) = &mut self.drawer_state {
-                des.drawer.content.settings.hide_values ^= true;
-                return Ok(CmdResult::Stay);
-            }
-        }
-
         if key == ENTER {
             self.close_drawer_input(false); // if there's an entry input
             if let DrawerCreation(PasswordInputState { input }) = &mut self.drawer_state {
@@ -513,29 +605,6 @@ impl AppState {
             }
         }
 
-        // --- input
-
-        if let Some(input) = self.drawer_state.input() {
-            if input.apply_key_event(key) {
-                if let DrawerEdit(des) = &mut self.drawer_state {
-                    if des.focus.is_search() {
-                        des.search.update(&des.drawer);
-                    }
-                }
-                return Ok(CmdResult::Stay);
-            }
-        }
-
-        // --- help
-
-        if key == F1 || key == QUESTION || key == SHIFT_QUESTION {
-            // notes:
-            //  - F1 is rarely available in terminals
-            //  - shift-? is here because on Windows on some keyboard I receive it for the ?
-            self.help = Some(HelpState::default());
-            return Ok(CmdResult::Stay);
-        }
-
         if let DrawerEdit(des) = &mut self.drawer_state {
             if key == HOME {
                 des.apply_scroll_command(ScrollCommand::Top);
@@ -586,7 +655,9 @@ impl AppState {
         if let DrawerEdit(des) = &mut self.drawer_state {
             if key == RIGHT {
                 match &des.focus {
-                    SearchEdit { previous_line } => {
+                    SearchEdit { previous_idx } => {
+                        let previous_line = previous_idx
+                            .and_then(|idx| des.entry_line(idx));
                         // we're here because apply_event on the input returned false,
                         // which means the right arrow key was ignored because it was
                         // at the end of the input. We'll assume the user wants to
@@ -611,7 +682,9 @@ impl AppState {
             if key == LEFT {
                 match &des.focus {
                     NameSelected { .. } => {
-                        des.focus = SearchEdit { previous_line: des.focus.line() };
+                        let previous_idx = des.focus.line()
+                            .and_then(|line| des.listed_entry_idx(line));
+                        des.focus = SearchEdit { previous_idx };
                     }
                     ValueSelected { line } => {
                         let line = *line;
@@ -634,33 +707,6 @@ impl AppState {
             }
         }
 
-        // --- other simple char shortcuts
-
-        if let Some(letter) = as_letter(key) {
-
-            if let DrawerEdit(des) = &mut self.drawer_state {
-                // if we're here, there's no input
-                match (letter, des.focus.line()) {
-                    ('n', _) => {
-                        // new entry
-                        des.search.clear();
-                        let idx = des.drawer.content.empty_entry();
-                        des.edit_entry_name_by_line(idx, EditionPos::Start);
-                    }
-                    ('d', Some(line)) => {
-                        // delete entry (with confirmation)
-                        des.focus = PendingRemoval { line };
-                    }
-                    ('/', _) => {
-                        // start searching
-                        des.focus = SearchEdit { previous_line: des.focus.line() };
-                    }
-                    _ => {}
-                }
-                return Ok(CmdResult::Stay);
-            }
-        }
-
         Ok(CmdResult::Stay)
     }
 }
