@@ -13,7 +13,7 @@ use {
 /// Contains the open drawer and an entry state.
 pub struct DrawerEditState {
     pub drawer: OpenDrawer,
-    pub scroll: usize, // number of lines hidden above the top of the view
+    pub scroll: usize, // number of listed entries hidden above the top of the view
     pub focus: DrawerFocus,
     edit_count: usize, // a counter to know whether the drawer changed
     pub search: SearchState,
@@ -131,9 +131,13 @@ impl DrawerEditState {
         }
     }
 
-
-    /// change the drawer drawing layout to adapt to the
-    /// content area in which it will be
+    /// update the drawer drawing layout.
+    ///
+    /// Must be called before every drawing as it depends on about everything:
+    /// - the content area in which it will be
+    /// - the filtering state
+    /// - the settings
+    /// - the entries
     pub fn update_drawing_layout(
         &mut self,
         content_area: &Area,
@@ -145,26 +149,42 @@ impl DrawerEditState {
         self.layout.lines_area.width = content_area.width;
         self.layout.lines_area.height = page_height;
         self.layout.name_width = (self.layout.lines_area.width / 3).min(30);
-        self.layout.value_height_addition = match &self.focus {
-            DrawerFocus::ValueSelected { line } => {
-                self.listed_entry_idx(*line).map_or(0, |idx| {
-                    // we compute the number of lines the text would be for
-                    // the available width (taking wrapping into account)
-                    let value_width = self.layout.value_width();
-                    let text = FmtText::from(
-                        termimad::get_default_skin(),
-                        &self.drawer.content.entries[idx].value,
-                        Some(value_width),
-                    );
-                    (text.lines.len().max(1) - 1)
-                        .min(page_height as usize - 4)
-                })
-            }
-            DrawerFocus::ValueEdit { input, .. } => {
-                input.content().line_count().min(page_height as usize - 4) - 1
-            }
-            _ => 0,
-        };
+        let dc = &self.drawer.content;
+        let open_all_values = dc.settings.open_all_values && !dc.settings.hide_values;
+        let value_width = self.layout.value_width();
+        let lines_count = self.listed_entries_count();
+        self.layout.content_height = 0;
+        self.layout.value_heights_by_line.clear();
+        for l in 0..lines_count {
+            let idx = self.listed_entry_idx(l).unwrap(); // SAFETY: we iter among valid lines
+            let height = match &self.focus {
+                DrawerFocus::ValueEdit { input, line } if l == *line => {
+                    // this line's value is edited, its height is given by the
+                    // number of lines computed by the input
+                    input.content().line_count().min(page_height as usize - 5)
+                }
+                _ => {
+                    let open = open_all_values || self.focus.is_value_selected(l);
+                    if open {
+                        // we compute the number of lines the text would be for
+                        // the available width, taking wrapping into account
+                        let text = FmtText::from(
+                            termimad::get_default_skin(),
+                            &self.drawer.content.entries[idx].value,
+                            Some(value_width),
+                        );
+                        text.lines.len().max(1).min(page_height as usize - 5)
+                    } else {
+                        // this line's value is neither open nor selected, we display
+                        // just the first line of the value (or a line of squares if
+                        // unselected values are hidden)
+                        1
+                    }
+                }
+            };
+            self.layout.content_height += height;
+            self.layout.value_heights_by_line.push(height);
+        }
         self.fix_scroll();
         self.layout.has_scrollbar = self.content_height() > self.page_height();
     }
@@ -173,19 +193,15 @@ impl DrawerEditState {
         self.search.update(&self.drawer)
     }
 
-    pub fn clicked_line(&self, y: u16) -> Option<usize> {
-        if y >= self.layout.lines_area.top {
-            let mut line = y as usize + self.scroll - self.layout.lines_area.top as usize;
-            if let Some(selected_line) = self.focus.line() {
-                if line > selected_line {
-                    if line <= selected_line + self.layout.value_height_addition {
-                        line = selected_line
-                    } else {
-                        line -= self.layout.value_height_addition;
-                    }
-                }
-            }
-            if line < self.content_height() {
+    pub fn clicked_line(&self, y: usize) -> Option<usize> {
+        let heights = self.layout.value_heights_by_line
+            .iter()
+            .enumerate()
+            .skip(self.scroll);
+        let mut sum_heights = self.layout.lines_area.top as usize;
+        for (line, height) in heights {
+            sum_heights += height;
+            if sum_heights > y {
                 return Some(line);
             }
         }
@@ -193,7 +209,29 @@ impl DrawerEditState {
     }
 
     pub fn scrollbar(&self) -> Option<(u16, u16)> {
-        self.layout.lines_area.scrollbar(self.scroll, self.content_height())
+        let content_height = self.layout.content_height;
+        let page_height = self.page_height();
+        if page_height >= content_height {
+            return None;
+        }
+        let heights = &self.layout.value_heights_by_line;
+        let real_hidden_before = heights[0..self.scroll].iter().sum::<usize>() as f64;
+        let real_from_scroll = heights[self.scroll..].iter().sum::<usize>() as f64;
+        let real_hidden_after = real_from_scroll - page_height as f64;
+        let hidden_before = if real_hidden_before > 0.0 {
+            (page_height as u16 * real_hidden_before as u16 / content_height as u16).max(1)
+        } else {
+            0
+        };
+        let hidden_after = if real_hidden_after > 0.0 {
+            (page_height as u16 * real_hidden_after as u16 / content_height as u16).max(1)
+        } else {
+            0
+        };
+        Some((
+            self.layout.lines_area.top + hidden_before,
+            self.layout.lines_area.bottom() - hidden_after
+        ))
     }
     fn page_height(&self) -> usize {
         self.layout.lines_area.height as usize
@@ -202,11 +240,6 @@ impl DrawerEditState {
         self.layout.clone()
     }
 
-    /// Give the additional height of the selected line due to
-    /// a selected value being several lines
-    pub fn value_height_addition(&self) -> usize {
-        self.layout.value_height_addition
-    }
     #[allow(dead_code)]
     pub fn current_cell(&self) -> Option<&str> {
         use DrawerFocus::*;
@@ -226,7 +259,21 @@ impl DrawerEditState {
             }
         }
     }
-    /// give the index of the entry from its line among the listed
+
+    pub fn entry_line(&self, idx: usize) -> Option<usize> {
+        if let Some(search_result) = &self.search.result {
+            for (line, matching_entry) in search_result.entries.iter().enumerate() {
+                if matching_entry.idx == idx {
+                    return Some(line);
+                }
+            }
+        } else if idx < self.drawer.content.entries.len() {
+            return Some(idx);
+        }
+        None
+    }
+
+    /// Give the index of the entry from its line among the listed
     /// entries (either all entries or only the ones matching if there's
     /// a search)
     pub fn listed_entry_idx(&self, line: usize) -> Option<usize> {
@@ -262,11 +309,10 @@ impl DrawerEditState {
             self.drawer.content.entries.len()
         }
     }
-    /// return the total height of the visible entries
+    /// return the total height of listed entries (the ones not filtered out)
     pub fn content_height(&self) -> usize {
-        self.listed_entries_count() + self.value_height_addition()
+        self.layout.content_height
     }
-
     /// Must be called on starting editing a name or value
     pub fn increment_edit_count(&mut self) {
         self.edit_count += 1;
@@ -279,7 +325,7 @@ impl DrawerEditState {
             warn!("internal error: edit count decremented when nul");
         }
     }
-    /// Tells whether the content was edited since opening
+    /// Tell whether the content was edited since opening
     /// (it may be equal)
     pub fn touched(&self) -> bool {
         match self.edit_count {
@@ -307,24 +353,28 @@ impl DrawerEditState {
     /// It's not necessary to call this other than from set_page_height
     /// (which is called before all drawings).
     fn fix_scroll(&mut self) {
-        let value_height_addition = self.value_height_addition();
-        let content_height = self.listed_entries_count() + value_height_addition;
+        let content_height = self.layout.content_height;
         let page_height = self.page_height();
+        let heights = &self.layout.value_heights_by_line;
         if content_height <= page_height {
             self.scroll = 0;
         } else if let Some(selection) = self.focus.line() {
-            if selection < 2 {
-                self.scroll = 0;
-            } else if self.scroll + 1 >= selection {
-                self.scroll = selection - 1;
+            if self.scroll >= selection {
+                self.scroll = selection;
             } else {
-                // TODO drink more coffee
-                while selection + value_height_addition + 1 >= self.scroll + page_height {
+                // let's ensure the end of the selection is visible
+                while heights[self.scroll..=selection].iter().sum::<usize>() > page_height {
                     self.scroll += 1;
                 }
+                // let's ensure there's not too much void at end
+                let last_entry = self.listed_entries_count() - 1;
+                while
+                    self.scroll > selection
+                    && heights[self.scroll-1..=last_entry].iter().sum::<usize>() >= page_height
+                {
+                    self.scroll -= 1;
+                }
             }
-        } else if self.scroll + page_height > content_height {
-            self.scroll = content_height - page_height;
         }
     }
     /// Save the drawer, which closes it, then reopen it, keeping the
@@ -378,7 +428,11 @@ impl DrawerEditState {
         }
     }
     pub fn apply_scroll_command(&mut self, scroll_command: ScrollCommand) {
-        self.scroll = scroll_command.apply(self.scroll, self.listed_entries_count(), self.page_height());
+        self.scroll = scroll_command.apply(
+            self.scroll,
+            self.listed_entries_count(),
+            self.page_height(),
+        );
     }
     pub fn close_input(&mut self, discard: bool) -> bool {
         if let DrawerFocus::NameEdit { line, input } = &self.focus {
@@ -415,16 +469,18 @@ impl DrawerEditState {
                 return true;
             }
         }
-        if let DrawerFocus::SearchEdit { previous_line } = self.focus {
-            self.focus = match self.best_search_line().or(previous_line) {
-                Some(line) => DrawerFocus::NameSelected { line },
-                None => DrawerFocus::NoneSelected,
-            };
+        if let DrawerFocus::SearchEdit { previous_idx } = self.focus {
             if discard {
                 // FIXME be back to previous focus ?
                 self.search.clear();
             }
             self.search.update(&self.drawer);
+            self.focus = self.best_search_line()
+                .or(previous_idx.and_then(|idx| self.entry_line(idx)))
+                .map_or(
+                    DrawerFocus::NoneSelected,
+                    |line| DrawerFocus::NameSelected { line },
+                );
             return true;
         }
         false
