@@ -11,7 +11,7 @@ pub enum DrawerFocus {
     ValueSelected { line: usize },
     NameEdit { line: usize, input: InputField },
     ValueEdit { line: usize, input: InputField },
-    SearchEdit { previous_line: Option<usize> },
+    SearchEdit { previous_idx: Option<usize> },
     PendingRemoval { line: usize },
 }
 
@@ -41,12 +41,21 @@ impl DrawerFocus {
             _ => false,
         }
     }
+    pub fn is_line_pending_removal(&self, entry_line: usize) -> bool {
+        match self {
+            Self::PendingRemoval { line } => *line == entry_line,
+            _ => false,
+        }
+    }
     pub fn is_value_selected(&self, entry_line: usize) -> bool {
         match self {
             Self::ValueSelected { line } => *line == entry_line,
             _ => false,
         }
     }
+    pub fn is_entry_edit(&self) -> bool {
+        matches!(self, DrawerFocus::NameEdit { .. } | DrawerFocus::ValueEdit { .. })
+    }
     /// return the input editing the name of the entry
     /// of given index, if it's currently edited
     pub fn name_input(&mut self, entry_line: usize) -> Option<&mut InputField> {
