@@ -1,6 +1,9 @@
-use crossterm::event::{
-    KeyCode::{self, *},
-    KeyEvent, KeyModifiers,
+use {
+    crossterm::event::{
+        KeyCode::{self, *},
+        KeyEvent,
+        KeyModifiers,
+    },
 };
 
 macro_rules! const_key {
@@ -33,13 +36,17 @@ const_key!(ESC, Esc);
 const_key!(HOME, Home);
 const_key!(LEFT, Left);
 const_key!(QUESTION, Char('?'));
+const_key!(SLASH, Char('/'));
+const_key!(D, Char('d'));
+const_key!(Y, Char('y'));
+const_key!(N, Char('n'));
 const_key!(SHIFT_QUESTION, Char('?'), KeyModifiers::SHIFT);
 const_key!(RIGHT, Right);
 //const_key!(SPACE, Char(' '));
 const_key!(TAB, Tab);
 const_key!(UP, Up);
 const_key!(PAGE_UP, PageUp);
-const_key!(F1, F(1));
+const_key!(CONTROL_A, Char('a'), KeyModifiers::CONTROL);
 const_key!(CONTROL_C, Char('c'), KeyModifiers::CONTROL);
 const_key!(CONTROL_V, Char('v'), KeyModifiers::CONTROL);
 const_key!(CONTROL_H, Char('h'), KeyModifiers::CONTROL);
@@ -52,8 +59,6 @@ const_key!(CONTROL_X, Char('x'), KeyModifiers::CONTROL);
 const_key!(CONTROL_UP, Up, KeyModifiers::CONTROL);
 const_key!(CONTROL_DOWN, Down, KeyModifiers::CONTROL);
 
-
-
 /// return the raw char if the event is a letter event
 pub fn as_letter(key: KeyEvent) -> Option<char> {
     match key {
@@ -64,3 +69,33 @@ pub fn as_letter(key: KeyEvent) -> Option<char> {
         _ => None,
     }
 }
+
+
+/// build a human description of a key event
+pub fn key_event_desc(key: KeyEvent) -> String {
+    let mut s = String::new();
+    if key.modifiers.contains(KeyModifiers::CONTROL) {
+        s.push('^');
+    }
+    if key.modifiers.contains(KeyModifiers::ALT) {
+        s.push('⌥');
+    }
+    match key.code {
+        Char('\r') | Char('\n') | Enter => {
+            s.push('⏎');
+        }
+        Char(c) if key.modifiers.contains(KeyModifiers::SHIFT) => {
+            s.push(c.to_ascii_uppercase());
+        }
+        Char(c) => {
+            s.push(c.to_ascii_lowercase());
+        }
+        F(u) => {
+            s.push_str(&format!("F{}", u));
+        }
+        _ => {
+            s.push_str(&format!("{:?}", key.code)); // FIXME check
+        }
+    }
+    s
+}
