@@ -1,5 +1,5 @@
 mkdocs build
-cp -r site/* ~/dev/www/dystroy/rhit/
+cp -r site/* ~/dev/www/dystroy/safecloset/
 
 # deploy on dystroy.org
 ~/dev/www/dystroy/deploy.sh
